using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using OpenTK.Windowing.Desktop;
using OpenTK.Windowing.Common;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace GraphicsManager;

public class ContextMenu : Window
{
    private static NativeWindowSettings set = new()
    {
        Title = nameof(ContextMenu),
        WindowBorder = WindowBorder.Hidden,
        APIVersion = new Version(3, 2),
        StartFocused = false,
        Size = new OpenTK.Mathematics.Vector2i(400, 5),
        StartVisible = false,
    };
    
    private FlowLayout fl;

    public ControlList Items => ((FlowLayout)(Controls[0])).Controls;

    public event Func<IRenderObject, Task>? OnItemClick; 

    public ContextMenu(int Width) : base(set)
    {
        this.Size = new(Width, 5);
        Controls.Add(fl = new FlowLayout(){Size = new(Width,5), Anchor = ObjectAnchor.All});
        Items.ControlAdded += ItemsOnControlAdded;
        BackgroundColor = new(0, 0, 0, 0);
    }

    private Task ItemsOnControlAdded(IRenderObject arg)
    {
        arg.Clicked += ArgOnClicked;
        this.Context.MakeCurrent();
        fl.Size = new(this.Size.X, arg.Location.Y + arg.Size.Y);
        this.Size = fl.Size;
        GL.Viewport(0,0, this.Size.X, this.Size.Y);
        return Task.CompletedTask;
    }

    protected override void OnResize(ResizeEventArgs e)
    {
        
    }

    public void HideContext(Window Parent)
    {
        if (Parent.ActiveMenu != this) return;
        Parent.ActiveMenu = null;
        Context.MakeCurrent();
        IsVisible = false;
    }

    public void ShowContext(Window Parent)
    {
        if (Parent.ActiveMenu == this)
        {
            Location = new((int)Parent.Location.X + (int)Parent.MousePosition.X, (int)Parent.Location.Y + (int)Parent.MousePosition.Y);
            Focus();
            return;
        }
        if (Parent.ActiveMenu is not null)
        {
            Parent.ActiveMenu.HideContext(Parent);
        }
        Parent.ActiveMenu = this;
        Context.MakeCurrent();
        Location = new((int)Parent.Location.X + (int)Parent.MousePosition.X, (int)Parent.Location.Y + (int)Parent.MousePosition.Y);
        IsVisible = true;
        GL.Viewport(0, 0, this.Size.X, this.Size.Y);
        DrawFrame();
    }

    private Task ArgOnClicked(IRenderObject arg)
    {
        if (OnItemClick is not null) _ = OnItemClick.Invoke(arg);
        return Task.CompletedTask;
    }
}