using GraphicsManager.Interfaces;

namespace GraphicsManager.Objects.Core;

public class ControlList
{
    private List<IRenderObject> _internal = new();

    public IRenderObject this[int Index] => _internal[Index];
    public IRenderObject this[uint Index] => _internal[(int)Index];

    public IEnumerable<IRenderObject> Where(Func<IRenderObject, bool> func) => _internal.Where(func);

    public int Length => _internal.Count;

    internal event Func<IRenderObject, Task>? ControlAdded;
    internal event Func<Task>? ControlRemoved;

    public void Remove(IRenderObject item)
    {
        _internal.Remove(item);
        item.Clean();
        if (ControlRemoved is not null) _ = ControlRemoved.Invoke();
    }
    
    public void Add(IRenderObject item)
    {
        if (ControlAdded is not null) ControlAdded.Invoke(item).Wait();
        _internal.Add(item);
    }
    
    public void Clear()
    {
        foreach (IRenderObject con in _internal)
        {
            con.Clean();
        }
        _internal.Clear();
        if (ControlRemoved is not null) _ = ControlRemoved.Invoke();
    }
}