﻿using System.Diagnostics;
using System.Reflection;
using SharpFont;

namespace GraphicsManager.Objects.Core;

public class Font
{
	private static List<Face>? System = null;
	private List<Face> _Faces = new();
	private Library lib;
	
	internal Font()
	{
		Name = null!;
		Assembly = null!;
		Embeded = false;
		if (lib is null) lib = new();
		//TODO add more systemfont dection methods
		if (System is null)
		{
			System = new();
			if (OperatingSystem.IsLinux())
			{
				try
				{
					Process proc = new()
					{
						StartInfo = new()
						{
							FileName = "/bin/bash",
							Arguments = "-c \"fc-list ':' file\"",
							RedirectStandardOutput = true,
						}
					};
					proc.Start();
					proc.WaitForExit();
					string[] files = proc.StandardOutput.ReadToEnd().Split($": {Environment.NewLine}");
					for (int i = 0; i < files.Length; i++)
					{
						System!.Add(new Face(lib, files[i], 0));
					}
				}
				catch
				{
				}
			}
		}

		for (int i = 0; i < System!.Count; i++) _Faces.Add(System![i]);
		//TODO add a reserved backup font
	}
	public IReadOnlyList<Face> Faces => _Faces.AsReadOnly();
	public void SetEmbeddedFont(string Font, Assembly? Assembly = null)
	{
		_ = Font ?? throw new ArgumentNullException(nameof(Font));
		string Base = "GraphicsManager.Resources.Fonts.";
		if (Assembly is not null) Base = string.Empty;
		byte[] f = (Assembly is null
			? Tools.GetResourceBytes(Base + Font)
			: Tools.GetResourceBytes(Assembly!, $"{Base}{Font}"));
		if (HasTopFont) _Faces[0] = new Face(lib, f, 0);
		else _Faces.Insert(0, new Face(lib, f, 0));
		HasTopFont = true;
		this.Assembly = Assembly;
		this.Embeded = true;
		this.Name = Font;
	}

	public static Font MakeEmbeddedFont(string Font, Assembly? Assembly = null)
	{
		_ = Font ?? throw new ArgumentNullException(nameof(Font));
		Font fontclass = new Font()
		{
			Assembly = Assembly,
			Embeded = true,
			Name = Font,
			HasTopFont = true
		};
		string Base = "GraphicsManager.Resources.Fonts.";
		if (Assembly is not null) Base = string.Empty;
		byte[] f = (Assembly is null
			? Tools.GetResourceBytes(Base + Font)
			: Tools.GetResourceBytes(Assembly!, $"{Base}{Font}"));
		fontclass._Faces.Insert(0, new Face(fontclass.lib, f, 0));
		return fontclass;
	}

	public static Font MakeFontFromSystem(uint PixelHeight = 20) => new Font() { PixelHeight = PixelHeight};

	public static Font MakeFontFromFile(string Font)
	{
		_ = Font ?? throw new ArgumentNullException(nameof(Font));
		Font fontclass = new Font()
		{
			Assembly = null,
			Embeded = false,
			Name = Font,
			HasTopFont = true
		};
		fontclass._Faces.Insert(0, new Face(fontclass.lib, Font, 0));
		return fontclass;
	}
	
	public void SetFontFile(string Font)
	{
		_ = Font ?? throw new ArgumentNullException(nameof(Font));
		if (HasTopFont) _Faces[0] = new Face(lib, Font, 0);
		else _Faces.Insert(0, new Face(lib, Font, 0));
		HasTopFont = true;
		this.Assembly = null;
		this.Embeded = false;
		this.Name = Font;
	}
	
	private bool HasTopFont = false;
	public uint PixelHeight { get; set; } = 20;
	public string Name { get; private set; } = default!;
	public bool Embeded { get; private set; }
	public Assembly? Assembly { get; private set; }
}
